
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"

#include "base/RndPermut.h"
#include "runtime/IOBuf.h"
#include "csm/ContentDbase.h"

#include "csm/ContentCfg.h"
#include "csm/InjectIter.h"
#include "csm/CdbBodyIter.h"

#include "xstd/gadgets.h"


CdbBodyIter::CdbBodyIter(): theCdb(0), theInjector(0), thePos(-1) {
}

void CdbBodyIter::cdb(ContentDbase *aCdb) {
	Assert(!theCdb && aCdb);
	theCdb = aCdb;
}

void CdbBodyIter::injector(InjectIter *anInjector) {
	Assert(!theInjector && anInjector);
	theInjector = anInjector;
}

void CdbBodyIter::reset() {
	Assert(!theInjector);
	BodyIter::reset();
	theCdb = 0;
	thePos = -1;
}

void CdbBodyIter::configure(WrBuf *aBuf, Size aContSize) {
	BodyIter::configure(aBuf, aContSize);
	Assert(theCdb && theCdb->count());
	Assert(thePos == -1);
	thePos = theRng(0, theCdb->count());
	if (theInjector)
		theInjector->start(this);
}

void CdbBodyIter::stop() {
	if (theInjector) {
		theInjector->creator()->putInjector(theInjector);
		theInjector = 0;
	}
}

void CdbBodyIter::pour() {
	CdbEntryPrnOpt opt;
	opt.buf = theBuf;
	opt.injector = theInjector;
	while (theBuiltSize < theContSize && !theBuf->full()) {
		const CdbEntry *e = theCdb->entry(thePos);
		const Size mark = theBuf->contSize();
		opt.sizeMax = sizeLeft();
		e->print(opt);
		theBuiltSize += theBuf->contSize() - mark;
		//cerr << here << "appended: " << sz << "; " << theBuiltSize << " <= " << theContSize << endl;
		thePos++;
		thePos %= theCdb->count();
	}
}
