
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "xstd/gadgets.h"
#include "pgl/PglRec.h"
#include "pgl/BenchSideSym.h"
#include "pgl/BenchSym.h"


BenchSym::BenchSym(): RecSym("Bench", new PglRec) {
	theRec->bAdd("rate", "peak_req_rate", 0);
	theRec->bAdd("BenchSide", "client_side", new BenchSideSym);
	theRec->bAdd("BenchSide", "server_side", new BenchSideSym);
	theRec->bAdd("BenchSide", "proxy_side", new BenchSideSym);
}

BenchSym::BenchSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool BenchSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Bench";
}

SynSym *BenchSym::dupe(const String &type) const {
	if (isA(type))
		return new BenchSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

bool BenchSym::peakReqRate(double &rate) const {
	return getRate("peak_req_rate", rate);
}

const BenchSideSym *BenchSym::clientSide() const {
	return getSide("client_side");
}

const BenchSideSym *BenchSym::serverSide() const {
	return getSide("server_side");
}

const BenchSideSym *BenchSym::proxySide() const {
	return getSide("proxy_side");
}

const BenchSideSym *BenchSym::side(const String &name) const {
	return getSide(name + "_side");
}

String BenchSym::clientHostCount(int &count) const {
	const BenchSideSym *side = clientSide();
	if (!side)
		return "client side of the bench is not configured";

	double peakRate;
	if (!peakReqRate(peakRate))
		return "peak_req_rate is not set for the bench";
	if (peakRate <= 0)
		return "peak_req_rate is not positive for the bench";

	double hostLoad;
	if (!side->maxHostLoad(hostLoad))
		return "max_host_load is not set for the client side of the bench";
	if (hostLoad <= 0)
		return "max_host_load is not positive for the client side of the bench";

	count = (int)xceil(peakRate, hostLoad);
	return 0;
}

const BenchSideSym *BenchSym::getSide(const String &name) const {
	SynSymTblItem *si = 0;
	Assert(theRec);
	Assert(theRec->find(name, si));
	if (!si->sym())
		return 0;

	const BenchSideSym *side = &(const BenchSideSym&)si->sym()->cast("BenchSide");
	side->bench(this);
	return side;
}
