
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglStringSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/MimeSym.h"


MimeSym::MimeSym(): RecSym("Mime", new PglRec) {
	theRec->bAdd("string", "type", 0);
	theRec->bAdd("string[]", "prefixes", 0);   // XXX: move to UrlGen
	theRec->bAdd("string[]", "extensions", 0); // XXX: move to UrlGen
}

MimeSym::MimeSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool MimeSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Mime";
}

SynSym *MimeSym::dupe(const String &type) const {
	if (isA(type))
		return new MimeSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

String MimeSym::mimeType() const {
	return getString("type");
}

bool MimeSym::extensions(Array<String*> &exts, RndDistr *&selector) const {
	SynSymTblItem *ei = 0;
	Assert(theRec->find("extensions", ei));
	if (!ei->sym())
		return false; // undefined

	// build extensions array and selector
	ArraySym &es = (ArraySym&)ei->sym()->cast("Array");
	selector = es.makeSelector("mime-extensions");
	for (int i = 0; i < es.count(); ++i) {
		const String &ext = ((const StringSym&)es[i]->cast("string")).val();
		exts.append(new String(ext));
	}

	return true;
}

bool MimeSym::prefixes(Array<String*> &pxs, RndDistr *&selector) const {
	SynSymTblItem *pi = 0;
	Assert(theRec->find("prefixes", pi));
	if (!pi->sym())
		return false; // undefined

	// build prefixes array and selector
	ArraySym &ps = (ArraySym&)pi->sym()->cast("Array");
	selector = ps.makeSelector("mime-prefixes");
	for (int i = 0; i < ps.count(); ++i) {
		const String &px = ((const StringSym&)ps[i]->cast("string")).val();
		pxs.append(new String(px));
	}

	return true;
}

