
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/h/iostream.h"

#include "xstd/Assert.h"
#include "xstd/gadgets.h"
#include "pgl/PglNetAddrBlock.h"


/* PglNetAddrBlock */

int PglNetAddrBlock::diffCount(const PglNetAddrBlock &b) const {
	if (isA("PglStrAddrBlock")) {
		if (b.isA("PglStrAddrBlock"))
			return ((const PglStrAddrBlock*)this)->
				countDiffs((const PglStrAddrBlock&)b);
		else
			return 2; // "big" difference
	}

	if (isA("PglRangeAddrBlock")) {
		if (b.isA("PglRangeAddrBlock"))
			return ((const PglRangeAddrBlock*)this)->
				countDiffs((const PglRangeAddrBlock&)b);
		else
			return 2; // "big" difference
	}

	Assert(false);
	return 2;
}

void PglNetAddrBlock::merge(PglNetAddrBlock &b) {
	if (isA("PglStrAddrBlock") && b.isA("PglStrAddrBlock")) {
		((PglStrAddrBlock*)this)->
			mergeWith((const PglStrAddrBlock&)b);
	} else
	if (isA("PglRangeAddrBlock") && b.isA("PglRangeAddrBlock")) {
		((PglRangeAddrBlock*)this)->
			mergeWith((const PglRangeAddrBlock&)b);
	} else {
		Assert(false);
	}
}


/* PglStrAddrBlock */

PglStrAddrBlock::PglStrAddrBlock(const char *aStart, const char *aStop):
	theStart(aStart), theStop(aStop) {
}

bool PglStrAddrBlock::isA(const String &type) const {
	return type == "PglStrAddrBlock";
}

PglNetAddrBlock *PglStrAddrBlock::clone() const {
	return new PglStrAddrBlock(theStart, theStop);
}

int PglStrAddrBlock::count() const {
	return 1;
}

int PglStrAddrBlock::countDiffs(const PglStrAddrBlock &b) const {
	return
		(theStop-theStart == b.theStop-b.theStart &&
		strncmp(theStart, b.theStart, theStop-theStart) == 0) ? 0 : 2;
}

void PglStrAddrBlock::mergeWith(const PglStrAddrBlock &b) {
	Assert(countDiffs(b) == 0);
}

bool PglStrAddrBlock::atLast() const {
	return true;
}

void PglStrAddrBlock::start() {
}

void PglStrAddrBlock::next() {
	Assert(false);
}

void PglStrAddrBlock::print(ostream &os) const {
	os.write(theStart, theStop-theStart);
}

void PglStrAddrBlock::printCur(ostream &os) const {
	print(os);
}


/* PglRangeAddrBlock */

PglRangeAddrBlock::PglRangeAddrBlock(int aStart, int aStop):
	theStart(aStart), theStop(aStop), thePos(aStart) {
}

bool PglRangeAddrBlock::isA(const String &type) const {
	return type == "PglRangeAddrBlock";
}

PglNetAddrBlock *PglRangeAddrBlock::clone() const {
	return new PglRangeAddrBlock(theStart, theStop);
}

int PglRangeAddrBlock::count() const {
	return theStop - theStart;
}

int PglRangeAddrBlock::countDiffs(const PglRangeAddrBlock &b) const {
	// exact match
	if (theStart == b.theStart && theStop == b.theStop)
		return 0;

	// can only merge without changing the number of addresses
	// if one range follows the other
	if (theStop == b.theStart || b.theStop == theStart)
		return 1;

	// "big" difference
	return 2;
}

void PglRangeAddrBlock::mergeWith(const PglRangeAddrBlock &b) {
	theStart = Min(theStart, b.theStart);
	theStop = Max(theStop, b.theStop);
}

bool PglRangeAddrBlock::atLast() const {
	return thePos == theStop-1;
}

void PglRangeAddrBlock::start() {
	thePos = theStart;
}

void PglRangeAddrBlock::next() {
	thePos++;
}

void PglRangeAddrBlock::print(ostream &os) const {
	os << theStart;
	if (theStart != theStop-1)
		os << '-' << (theStop-1);
}

void PglRangeAddrBlock::printCur(ostream &os) const {
	os << thePos;
}
