
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PGL_NET_ADDRESS_LEXER_H
#define PGL_NET_ADDRESS_LEXER_H

#include "xstd/Ring.h"
#include "xstd/String.h"

class PglNetAddrRange;

// parsers an address specification into lexems
// and facilitates conversion of the lexems into 
// PglNetAddrRange blocks
class PglNetAddrLexer {
	protected:

		class Lexem {
			public:
				Lexem();
				Lexem(const char *aStart, const char *aStop);

				char tag() const { return *theStart; }
				const char *start() const { return theStart; }
				const char *stop() const { return theStop; }

			protected:
				const char *theStart; // pointers to the string
				const char *theStop;  // being parsed
		};

	public:
		PglNetAddrLexer(PglNetAddrRange *anOwner, const String &anImage);

		operator void *() const { return theLexems.empty() ? 0 : (void*)-1; }

		bool parse(); // split the image into lexems

		// first char of the next lexem
		char next() const { return charAt(0); }
		// converts next lexem into a block
		void step();
		void skip(int count = 1);  // skips next count lexems

		bool range(); // tries to convert next lexems into a range block

	protected:
		char charAt(int pos) const;
		int intAt(int pos) const;

		bool rangeItem();

	protected:
		Ring<Lexem> theLexems;
		PglNetAddrRange *theOwner;
		const String theImage; // XXX: check if needed
};

#endif
