
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/h/math.h"
#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"

#include "xstd/String.h"
#include "xstd/gadgets.h"
#include "pgl/PglRec.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglNetAddrSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglNetAddrRange.h"
#include "pgl/PglNetAddrArraySym.h"
#include "pgl/VerFourAsSym.h"
#include "pgl/BenchSym.h"
#include "pgl/BenchSideSym.h"


VerFourAsSym::VerFourAsSym(const String &aType, PglRec *aRec): AddrSchemeSym(aType, aRec) {
	theRec->bAdd("int", "agents_per_addr", 0);
}

bool VerFourAsSym::agentsPerAddr(int &apa) const {
	return getInt("agents_per_addr", apa);
}

String VerFourAsSym::addresses(const BenchSideSym *side, int xOffset, ArraySym *&addrs) const {
	if (!side)
		return "bench side is not specified";

	const NetAddrSym *mask = side->addrMaskSym();
	if (!mask)
		return "addr_mask not set";

	Assert(side->bench());
	double reqRate, hostLoad, agentLoad;
	int addrLoad;
	if (!side->bench()->peakReqRate(reqRate) || 
		!side->maxHostLoad(hostLoad) || 
		!side->maxAgentLoad(agentLoad) || agentLoad <= 0 ||
		!this->agentsPerAddr(addrLoad) || addrLoad <= 0)
		return "peak_req_rate, max_host_load, max_agent_load or agents_per_addr not set";

	const int maxAgentsPerHost = (int)xceil(hostLoad, agentLoad);
	const int maxAddrPerHost = (int)xceil(maxAgentsPerHost, addrLoad);

	// find min subnet that can fit maxAddrPerHost addresses
	int subnet = 25;
	int maxAddrPerSnet = -1;
	for (int i = 1; maxAddrPerSnet < maxAddrPerHost && i <= 128; i *= 2) {
		maxAddrPerSnet = i * 250;
		--subnet;
	}
	if (maxAddrPerSnet < maxAddrPerHost || subnet == 25)
		return "not enough IP addresses";

	// find the number of agents 
	const int hostCnt = (int)xceil(reqRate, hostLoad);
	const int agentCnt = doubleDiv(hostCnt, reqRate, agentLoad);
	const int agentPerHost = agentCnt / hostCnt;
	if (agentCnt > addrLoad * maxAddrPerHost * hostCnt)
		return "too many agents?";

	// distribute agentCnt agents among subnets (upto maxAddrPerSnet each)
	ArraySym *ips = new NetAddrArraySym();
	for (int s = 0; ips->count() < agentCnt; ++s) {
		// one subnet, one host
		int x = s * maxAddrPerSnet/250;
		int y = 1;
		for (int a = 0; a < agentPerHost;) { // agents
			// one address per addrLoad agents
			if (x >= 128)
				return "request rate too high";
			NetAddrSym nas;
			makeAddrSym(*mask, x+xOffset, y, subnet, nas);

			// add addrLoad agents for one address
			for (int apa = 0; apa < addrLoad && a < agentPerHost; ++apa, ++a)
				ips->add(nas);

			if (++y == 251) {
				++x;
				y = 1;
			}
		}
	}

	addrs = ips;
	return 0;
}

