
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PGL_SYMBOLS_H
#define PGL_SYMBOLS_H

#include "xparser/TokenSym.h"
#include "pgl/PglRec.h"
#include "pgl/PglExprSym.h"

// various language symbols


// template to define simple symbols
#define SimpleSynSym(Type) \
	struct Type##Sym: public SynSym { \
		Type##Sym(): SynSym(#Type) {} \
		Type##Sym(const char *aType): SynSym(aType) {} \
	};

SimpleSynSym(Code)
SimpleSynSym(StatementSeq)
SimpleSynSym(Statement)
SimpleSynSym(SimpleStatement)
SimpleSynSym(BlockStatement)
SimpleSynSym(DeclStatement)
SimpleSynSym(ExprStatement)
SimpleSynSym(InitDecl)
SimpleSynSym(Assignment)
SimpleSynSym(PartAssign)
SimpleSynSym(BlockOpen)
SimpleSynSym(BlockClose)


class SynSymTblItem;

struct PureDeclSym: public SynSym {
	SynSymTblItem *theDecl;
	PureDeclSym(SynSymTblItem *aDecl): SynSym("PureDecl"), theDecl(aDecl) {}
	// does not destroy declaration
};

// "empty" or "no op" expression -- does not change LHS value
class NopExprSym: public ExpressionSym {
	public:
		NopExprSym(): ExpressionSym("NopExpr") {}
};

// "undefined" or "null" expression -- undefines LHS value
class UndefOpExprSym: public ExpressionSym {
	public:
		UndefOpExprSym();

		virtual bool isA(const String &type) const;
};

// function call
class CallSym: public ExpressionSym {
	public:
		CallSym(): ExpressionSym("Call") {}
};

#define TypedTokenSym(Type) \
	class Type##Sym: public TokenSym { \
		public: \
			Type##Sym(const String &s): TokenSym(s) { type(#Type); } \
		\
		protected: \
			virtual SynSym *dupe(const String &type) const { \
				if (isA(type)) return new Type##Sym(spelling()); \
				else return TokenSym::dupe(type); \
			} \
	};

TypedTokenSym(ObjName);
TypedTokenSym(KnownCtxName);
TypedTokenSym(CtxName);
TypedTokenSym(Type);
TypedTokenSym(UnOper);
TypedTokenSym(BinOper);

#endif
