
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_BROADCAST_RECEIVER_H
#define PG_BROADCAST_RECEIVER_H

#include "xstd/Array.h"

class BcastChannel;
class Connection;
class Xaction;
class IcpXaction;
class OLog;

// broadcast channel(s) subscriber

class BcastRcver {
	public:
		typedef enum { ieNone, ieWssFill, ieWssFreeze } InfoEvent;

	public:
		virtual ~BcastRcver();

		inline void noteEvent(BcastChannel *ch, const Connection *c);
		inline void noteEvent(BcastChannel *ch, const Xaction *x);
		inline void noteEvent(BcastChannel *ch, const IcpXaction *x);
		inline void noteEvent(BcastChannel *ch, const char *msg);
		inline void noteEvent(BcastChannel *ch, const Error &err);
		inline void noteEvent(BcastChannel *ch, InfoEvent ev);
		inline void noteEvent(BcastChannel *ch, OLog *log);

		void startListen();
		void stopListen();

	protected:
		virtual void noteConnEvent(BcastChannel *ch, const Connection *c);
		virtual void noteXactEvent(BcastChannel *ch, const Xaction *x);
		virtual void noteIcpXactEvent(BcastChannel *ch, const IcpXaction *x);
		virtual void noteMsgStrEvent(BcastChannel *ch, const char *msg);
		virtual void noteErrEvent(BcastChannel *ch, const Error &err);
		virtual void noteInfoEvent(BcastChannel *ch, InfoEvent ev);
		virtual void noteLogEvent(BcastChannel *ch, OLog &log);

	protected:
		Array<BcastChannel*> theChannels; // channels we are subscribed to
};


/* inlined methods */

inline
void BcastRcver::noteEvent(BcastChannel *ch, const Connection *c) {
	noteConnEvent(ch, c);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, const Xaction *x) {
	noteXactEvent(ch, x);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, const IcpXaction *x) {
	noteIcpXactEvent(ch, x);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, const char *msg) {
	noteMsgStrEvent(ch, msg);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, const Error &err) {
	noteErrEvent(ch, err);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, InfoEvent ev) {
	noteInfoEvent(ch, ev);
}

inline
void BcastRcver::noteEvent(BcastChannel *ch, OLog *log) {
	noteLogEvent(ch, *log);
}

#endif
