
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_IO_BUFFER_H
#define PG_IO_BUFFER_H

#include "xstd/LibInit.h"

class IOBuf {
	public:
		IOBuf();
		~IOBuf() { if (theBuf) zip(); }

		void reset() { theOutOff = theInOff = 0; if (theBuf) zip(); }

		bool empty() const { return theOutOff >= theInOff; }
		bool full() const { return theInOff >= theCapacity; }

		char *space() { if (!theBuf) unzip(); return theBuf + theInOff; }
		const char *content() const { if (theBuf) return theBuf + theOutOff; Assert(empty()); return 0; }

		Size spaceSize() const { return theCapacity - theInOff; }
		Size contSize() const { return theInOff - theOutOff; }

		void appendRnd(Size sz);     // random content
		Size appendRndUpTo(Size sz); // random content
		void appended(Size sz) { theInOff += sz; Assert(theInOff <= theCapacity); if (!theBuf) unzip(); }
		void consumed(Size sz) { theOutOff += sz; if (theOutOff == theInOff) reset(); else Assert(theOutOff < theInOff); }
		void consumedAll() { reset(); }
		void pack();

	protected:
		void zip();
		void unzip();

	protected:
		char *theBuf;
		Size theCapacity;
		Size theOutOff;
		Size theInOff;
};


class RdBuf: public IOBuf {
};

class WrBuf: public IOBuf {
	public:
		void append(const char *buf, Size sz);
		void overwrite(int offset, const char *buf, Size sz);
};


LIB_INITIALIZER(IOBufInit)

#endif
