
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/rndDistrs.h"
#include "base/RndPermut.h"
#include "runtime/PopModel.h"
#include "csm/ContentCfg.h"
#include "csm/ContentMgr.h"
#include "pgl/ServerSym.h"
#include "server/SrvCfg.h"
#include "server/srvRepTypes.h"

SrvCfg::SrvCfg(): theServer(0), thePopModel(0), theRepTypeSel(0) {
}

SrvCfg::~SrvCfg() {
	delete thePopModel;
	delete theRepTypeSel;
}

void SrvCfg::configure(const ServerSym *aServer) {
	Assert(!theServer && aServer);
	theServer = aServer;

	if (PopModelSym *pms = theServer->popModel()) {
		thePopModel = new PopModel;
		thePopModel->configure(pms);
	}

	configureRepTypes();

	Array<ContentSym*> csyms;
	Array<double> cprobs;
	Assert(theServer->contents(csyms, cprobs) && csyms.count());
	Array<ContentCfg*> ccfgs;
	TheContentMgr.get(csyms, ccfgs);
	ContentTypeIdx::configure(ccfgs);
}

void SrvCfg::configureRepTypes() {
	theRepTypeSel = theServer->repTypes(SrvRepTypeIdx());
	if (!theRepTypeSel)
		theRepTypeSel = new ConstDistr(new RndGen, rptBasic); // default
	theRepTypeSel->rndGen(LclRndGen("server_req_types"));
}


/* SrvSharedCfgs */

SrvCfg *SrvSharedCfgs::getConfig(const ServerSym *rs) {
	for (int i = 0; i < count(); ++i) {
		if (item(i)->theServer == rs)
			return item(i);
	}
	return addConfig(rs);
}

SrvCfg *SrvSharedCfgs::addConfig(const ServerSym *rs) {
	SrvCfg *cfg = new SrvCfg;
	cfg->configure(rs);
	append(cfg);
	return cfg;
}

