
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include <fstream.h>
#include "xstd/h/iomanip.h"

#include "xstd/gadgets.h"
#include "base/CmdLine.h"
#include "base/opts.h"
#include "pgl/PglNetAddrRange.h"


class MyOpts: public OptGrp {
	public:
		MyOpts():
			theHelpOpt(this,     "help",         "list of options"),
			theDomainStr(this,   "domain <str>", "domain"),
			theAddrStr(this,     "addrs <str>",  "addresses"),
			theNameStr(this,     "names <str>",  "names")
			{}

		virtual bool validate() const;

		//virtual ostream &printAnonym(ostream &os) const;
		//virtual bool parseAnonym(const Array<const char *> &opts);
		//virtual bool canParseAnonym() const { return true; }

	public:
		HelpOpt theHelpOpt;
		StrOpt theDomainStr;
		StrOpt theAddrStr;
		StrOpt theNameStr;
};


static MyOpts TheOpts;

/* MyOpt */

bool MyOpts::validate() const {
	if (!theDomainStr)
		cerr << "must specify the domain" << endl;
	else
	if (!theAddrStr || !theNameStr)
		cerr << "must specify addresses and domain names" << endl;
	else
		return true;
	return false;
}

static
void configureLogs(int prec) {
	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

static
void configure() {
	configureLogs(2);
}

static
void printSoa(ostream &os) {
	const String serialNo = "20000833"; // XXX;
	os << "@ IN SOA "
		<< TheOpts.theDomainStr << '.' // XXX: the host on which this data file resides
		<< " root." << TheOpts.theDomainStr << '.' // person responsible
		<< " (" << endl
		<< "\t " << serialNo << ";  Serial" << endl
		<< "\t 10800   ; Refresh" << endl
		<< "\t 1801    ; Retry" << endl
		<< "\t 3600000 ; Expire" << endl
		<< "\t 259200  ; Minimum" << endl
		<< ")" << endl
		<< endl;

	// also add NS records
	os << "  NS " << TheOpts.theDomainStr << '.' << endl; // XXX

	os << endl;
}

static
String reverseAddr(const String &addr) {
	String res = "";
	String v = addr;

	while (v) {
		if (const char *dot = v.rchr('.')) {
			res += dot;
			v = v(0, dot - v);
		} else {
			res += '.';
			res += v;
			v = 0;
		}
	}

	return res(1, res.len()+1);
}

int main(int argc, char *argv[]) {

	CmdLine cmd;
	cmd.configure(Array<OptGrp*>() << &TheOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate())
		return -1;

	configure();

	PglNetAddrRange addrParser;
	PglNetAddrRange nameParser;

	String s = TheOpts.theAddrStr;

	if (!addrParser.parse(s))
		cerr << here << "malformed address range: `" <<  TheOpts.theAddrStr << "'" << endl << xexit;

	if (!nameParser.parse(TheOpts.theNameStr))
		cerr << here << "malformed name range: `" <<  TheOpts.theNameStr << "'" << endl << xexit;

	Array<String*> addrs;
	Array<String*> names;

	addrParser.toStrs(addrs);
	nameParser.toStrs(names);

	// 1:1 mapping only, for now
	Must(addrs.count() == names.count());

	const String dirFname(TheOpts.theDomainStr);
	const String revFname(TheOpts.theDomainStr + ".rev");
	ofstream direct(dirFname);
	ofstream reverse(revFname);

	const String commonAddr = "192.168"; // XXX
	const String commonName = TheOpts.theDomainStr;

	// SOA records
	printSoa(direct);
	printSoa(reverse);

	// A and PTR records
	for (int i = 0; i < addrs.count(); ++i) {

		direct
			<< *names[i] << '.'
			<< " IN A "
			<< *addrs[i]
			<< endl;

		reverse 
			<< reverseAddr(*addrs[i]) << ".IN-ADDR.ARPA."
			<< " IN PTR "
			<< *names[i] << '.' 
			<< endl;
	}

	cout << "# zone descriptions for in named.conf: " << endl
		<< "zone \"" << reverseAddr(commonAddr) << ".IN-ADDR.ARPA\" {" << endl
		<< "\ttype master;" << endl
		<< "\tfile \"master/" << revFname << "\";" << endl
		<< "};" << endl
		<< endl
		<< "zone \"" << commonName << "\" {" << endl
		<< "\ttype master;" << endl
		<< "\tfile \"master/" << dirFname << "\";" << endl
		<< "};" << endl
		<< endl
		;

	return 0;
}
